from typing import Literal

from powerpwn.nocodemalware.enums.command_to_run_enum import CommandToRunEnum
from powerpwn.nocodemalware.models.cmd_arguments import CommandArguments
from powerpwn.nocodemalware.models.exflirtate_file_args_properties import ExflirtateFileArgsProperties


class ExflirtateFileCommandArgs(CommandArguments[ExflirtateFileArgsProperties]):
    command_to_run: Literal[CommandToRunEnum.EXFILTRATION] = CommandToRunEnum.EXFILTRATION
