CACHE_PATH = ".dump"
DATA_MODEL_FILE_EXTENSION = ".json"
SPEC_JWT_NAME = "ApiHubBearerAuth"
DATA_MODEL_VERSION = "0.0.1"
POWER_APPS_SCOPE = "https://service.powerapps.com/.default"
API_HUB_SCOPE = "https://apihub.azure.com/.default"
AZURE_CLI_APP_ID = "04b07795-8ddb-461a-bbee-02f9e1bf7b46"
GRAPH_API_SCOPE = "https://graph.microsoft.com/.default"
ENCODING = "UTF8"
