from typing import Literal

from powerpwn.nocodemalware.enums.command_to_run_enum import CommandToRunEnum
from powerpwn.nocodemalware.models.cmd_arguments import CommandArguments
from powerpwn.nocodemalware.models.code_exec_args_properties import CodeExecArgsProperties


class CodeExecCommandArgs(CommandArguments[CodeExecArgsProperties]):
    command_to_run: Literal[CommandToRunEnum.CODE_EXEC] = CommandToRunEnum.CODE_EXEC
