{% extends "base.html" %}

{% block content %}

<pre id="json-obj"></pre>
{% endblock %}

{% block scripts %}
<script>
    var data = JSON.parse({{ json_object | tojson | safe}});
    document.getElementById("json-obj").innerHTML = JSON.stringify(data, null, 4);
  </script>
{% endblock %}
